program diagmet

!MOD: Read top prec instead of ls and convective  
!MOD: write  potential  temperuture for plume rise   in netcdf file 
! 15/12/2008
  use diagmet_common
  use diagmet_science
  use chimere_consts
  use chimere_params  
  use netcdf
#ifdef IFORT
  use ifport
  use ifposix
#endif
  implicit none

#define NCERR(lnum) if(ncstat/=NF90_NOERR) call nc_err(ncstat,lnum,'diagmet.f90')

#define NFCHAR NF90_CHAR
#define NFLOAT NF90_FLOAT

  !  Input arguments
  
  real,parameter :: cflmax=0.5

  integer            :: nhourrun    ! Model run duration
  character(len=132) :: fniMETEO    ! netCDF meteo file from exdom
  character(len=132) :: fniLDU      ! Landuse file
  character(len=132) :: fniLAP      ! Land parameters file
  character(len=132) :: fniVCO      ! File containing the vertical hybrid coefficients
  character(len=132) :: fniMETINFOS ! Input meteo information file
  character(len=132) :: fnoMET      ! Output meteo data
  integer,allocatable,dimension(:) :: nphourm     ! Theorical nphour to avoid CFL calculated at each hour
  integer,allocatable,dimension(:) :: nphourm_int ! Theorical nphour to avoid CFL between two hours , this
                                                  ! parameter is transfered to CHIMERE

  namelist /args/ nhourrun,fniMETEO,fniLDU,fniLAP,fniVCO,fniMETINFOS,fnoMET

  !  Local constants
  character(len=132),parameter :: fniARGS = 'diagmet.nml'

  !  Constant netCDF attributes
  character(len=*),parameter :: title=      'CHIMERE SUITE'
  character(len=*),parameter :: subtitle=   'Meteo file'
  character(len=*),parameter :: generator=  'Generated by diagmet'
  character(len=*),parameter :: conventions=''


  !****** types and variables
  !  Array dimensions, dynamic
  integer           :: nlevels    ! nb of levels in input netCDF file
  integer           :: nlayers    ! Number of levels in the output netCDF file
  integer           :: dl         ! length of MM5 date string, according to input file


  ! unit numbers of ASCII and binary files
  integer :: ifn_args
  integer :: ifn_minf
  integer :: ifn_ldus
  integer :: ifn_ldpa
  integer :: ifn_vcoo
  integer :: ifn_tst

  ! Local arrays
  real,allocatable,dimension(:,:,:) :: fland ! Land use fractions
  character(len=dlen) :: datebuf       ! to hold current date in MM5 format


  real :: czen          ! cosine of solar zenith angle
  integer :: itim       ! index of main time loop
  integer :: ime,izo,ive    ! nmerid and nzonal indexes of grid points 
  integer :: ilan       ! index for land types
  integer :: ilay       ! index for chimere layers
  integer :: im         ! month index for land parameters

  ! netcdf stuff
  ! m_ refers to exdom output (diagmet input)
  ! d_ refers to diagmet output
  integer :: m_id,d_id                       ! IDs of input,output files
  integer :: ncstat                          ! return code for netCDF functions
  integer :: m_time_dimid,d_time_dimid       ! Time dimension IDs
  integer :: m_date_dimid,d_date_dimid       ! Date string dimension ID
  integer :: m_zonal_dimid,d_zonal_dimid     ! WE dimension IDs
  integer :: m_merid_dimid,d_merid_dimid     ! SN dimension IDs
  integer :: m_levels_dimid,d_layers_dimid   ! BT dimension IDs
  ! Input/Output variables IDs
  integer :: m_lon_varid,m_lat_varid, d_lon_varid,d_lat_varid
  integer :: d_avcoord_varid,d_bvcoord_varid
  integer :: d_nphourm_varid
  integer :: m_times_varid,d_times_varid
  integer :: m_alti_varid, d_alti_varid
  integer :: m_pres_varid
  integer :: m_temp_varid, d_temp_varid
  integer :: m_sphu_varid, d_sphu_varid
  integer :: m_winz_varid, d_winz_varid
  integer :: m_winm_varid, d_winm_varid
  integer :: m_winw_varid, d_winw_varid
  integer :: m_usta_varid, d_usta_varid
  integer :: m_sshf_varid
  integer :: m_slhf_varid
  integer :: m_hght_varid, d_hght_varid
  integer :: m_clol_varid
  integer :: m_clom_varid
  integer :: m_cloh_varid
  integer :: m_cliq_varid
  integer :: m_copc_varid
  integer :: m_lspc_varid
  integer :: m_cice_varid
  integer :: m_rain_varid
  integer :: m_tem2_varid, d_tem2_varid
  integer :: m_u10m_varid
  integer :: m_v10m_varid
  integer :: m_soim_varid, d_soim_varid
  integer :: m_swrd_varid, d_swrd_varid   ! SW radiation for MEGAN
  ! Output only variables IDs
  integer :: d_airm_varid
  integer :: d_kzzz_varid
  integer :: d_clwc_varid
  integer :: d_atte_varid
  integer :: d_aerr_varid
  integer :: d_obuk_varid
  integer :: d_wsta_varid
  integer :: d_sreh_varid
  integer :: d_topc_varid
  integer :: d_w10m_varid
  integer :: d_w10s_varid
  integer :: d_dpeu_varid
  integer :: d_dped_varid
  integer :: d_dpdu_varid
  integer :: d_dpdd_varid
  integer :: d_theta_varid


  integer :: m_times                                    ! number of records in input file
  integer,allocatable,dimension(:) :: d_dimids          ! dimensions vector
  integer,allocatable,dimension(:) :: m_stvec,d_stvec   ! start vectors for R/W functions
  integer,allocatable,dimension(:) :: m_cntvec,d_cntvec ! count vectors for R/W functions
  character(len=1024) :: history,history_in
  integer :: hin_len
  
  ! misc
  integer :: idnow      ! current date,expected
  integer :: jyear,jmon,jday,jhour
  integer :: ierr       ! return code for non-netCDF I/O
  integer :: im1        ! placeholder to read month from land parameter file
  integer :: np
  
  real    :: layerdepth

  ! System information stuff
  character(len=32) :: systime
  character(len=64) :: hname
  character(len=32) :: usrname
  character(len=255) :: cwd

  logical :: usta_req
  logical :: sshf_req
  logical :: slhf_req
  logical :: pblh_req
  logical :: clol_req
  logical :: clom_req
  logical :: cloh_req
  logical :: cice_req
  logical :: rain_req
  logical :: soim_req
  logical :: swrd_req  ! SW radiation for MEGAN
  logical :: u10m_req
  logical :: v10m_req

  logical,parameter :: times_man = .true.
  logical,parameter :: alti_man  = .true.
  logical,parameter :: pres_man  = .true.
  logical,parameter :: temp_man  = .true.
  logical,parameter :: mixr_man  = .true.
  logical,parameter :: zwin_man  = .true.
  logical,parameter :: mwin_man  = .true.
  logical,parameter :: tem2_man  = .true.

  ! functions
  real,external :: cosineza
  
  ! parameters for CFL
  integer :: cumnphour
  real :: dxoveru,dxoverv,dzoverw,dtmin
  real :: dtminu,dtminv,dtminw,tpercent
  real :: dtminup_eu,dtminup_ed,dtminup_du,dtminup_dd
  real :: dzovereu,dzoverdd,dzoverdu,dzovered
 
!=============================================================================
  integer::ifn_theta
!#ifdef DEBUG_DIAGMET
  call opfi(ifn_tst,'METEO','f','n')
!#endif
  
  ! Read input arguments
  call opfi(ifn_args,fniARGS,'f','o')
  read(ifn_args,nml=args,iostat=ierr)
  if(ierr/=0) then
     print *,'diagmet: problem reading arguments namelist'
     stop
  endif
  close(ifn_args)

  nlayers = nvert_raw

  ! Get various system informations
  call get_system(usrname,hname,systime,cwd)

  ! Reading input information
  call get_options

  ! Terrain files
  call opfi(ifn_ldus,fniLDU,'f','o')
  call opfi(ifn_ldpa,fniLAP,'f','o')
  call opfi(ifn_vcoo,fniVCO,'f','o')

!  open theta
!   call opfi(ifn_theta,fnoTHETA,'u','n')

  ! Open meteo netCDF input file from exdom
  ncstat=nf90_open(fniMETEO,NF90_NOWRITE,m_id)
  NCERR(__LINE__)

  ! Read "history" attribute from input file
  call read_atts

  ! Open meteo netCDF output file to chimere
  ncstat=nf90_create(fnoMET,NF90_CLOBBER,d_id)
  NCERR(__LINE__)

  ! Verify that all required (mandatory and optional) variables
  ! are contained in the input netCDF file
  call verif_req

  ! Reading dimensions from netCDF input file
  call read_dims

  ! allocate memory for input and local fields
  call alloc_diagmet_locals

  ! Allocating memory for output fields
  call alloc_diagmet_globals

  ! Creating dimensions in output file
  call create_dims

  ! Defining variables in output files
  call define_out_vars

  ! Create attributes
  call create_global_atts

  !! End netCDF definition mode
  ncstat=nf90_enddef(d_id)

  ! Reading coordinates from input netCDF file
  ncstat=nf90_get_var(m_id,m_lon_varid,xlong)
  NCERR(__LINE__)
  ncstat=nf90_get_var(m_id,m_lat_varid,xlati)
  NCERR(__LINE__)

  ! Reading land use fractions
  do ime=1,nmerid
     do izo=1,nzonal
        read(ifn_ldus,*,iostat=ierr)(fland(izo,ime,ilan),ilan=1,nlduse)
        if (ierr/=0) stop '*** diagmet : error reading Land Use file'
     enddo
  enddo

  ! Reading land parameters
  read(ifn_ldpa,*,iostat=ierr)
  do im=1,nmonth
     read(ifn_ldpa,*,iostat=ierr)im1,(zom(ilan,im),ilan=1,nlduse)
     if (ierr/=0) stop '*** diagmet : error reading Land Parameters file'
  enddo

  ! Reading vertical hybrid coefficients
  do ilay=1,nlayers
     read(ifn_vcoo,*,iostat=ierr) cac(ilay),cbc(ilay)
     if (ierr/=0) stop '*** diagmet: error reading Vertical Hybrid Coefficients file'
  enddo

  ! Write coordinates
  ncstat=nf90_put_var(d_id,d_lon_varid,xlong)
  NCERR(__LINE__)
  ncstat=nf90_put_var(d_id,d_lat_varid,xlati)
  NCERR(__LINE__)
  ncstat=nf90_put_var(d_id,d_avcoord_varid,cac)
  NCERR(__LINE__)
  ncstat=nf90_put_var(d_id,d_bvcoord_varid,cbc)
  NCERR(__LINE__)
  
! calculation of the model geometry [for deep conv. -lmbb]
  call gridsize
  
  print*,'Number of raw meteo levels    : ',nlevels
  print*,'Number of chimere meteo levels: ',nlayers
  print *
!                  1234567890123456712345671234567123456712345671234567123456712345671234567
  write (*,'(a)') "      scad  ustar   potf  wstar   hmix  atten  optical depth L,M,H topcld"
  cumnphour=0

  ! Loop on time
!######################
  do itim=1,nhourrun+1

     ! Date
     call reldat(opt%idstart,itim-1,idnow)
     call ddate(idnow,jyear,jmon,jday,jhour)

     write (90,*)
     write (90,'(a,i3,a,i10,2(a,i5))') "Elaboro istante ",itim,": ",idnow, &
       " nlev ",nlevels," nshow ",opt%nsho

     ! Initialization of some optional variables
     call init_optional
    
     ! Reading mandatory and optional data
     call read_data

     ! Date check
     call check_date
     
     ! Copying unmodified input fields to output fields
     d_tem2 = m_tem2
     d_w10m = sqrt(m_u10m*m_u10m + m_v10m*m_v10m)
     d_soim = m_soim
     ! SW radiation for MEGAN
     d_swrd = m_swrd
!  Calculation of the velocity divergence, horizontal and vertical transport
     call windiv

! Loop on grid points
     np=0
     do ime=1,nmerid
        do izo=1,nzonal
           np=np+1

!  Calculation of the evaporation of water at the surface [for deep conv -lmbb]
!  unit: W/m2   /  [J Kg^{-1}] with 1 watt = 1 J.s-1
!  Lv = 2.45e6[J Kg^{-1}].
!  evapo = kg * m-2 * s-1

	    evapo=m_slhf(izo,ime)/Lv
	    
           ! Definition of the 1D local column basic variables
           ! Calculation of additional column variables
           call defcolumn(izo,ime)

           ! Low cloud top and rhmaxx
           call low_cloud_top

           ! CHIMERE Layer top altitudes
           call layer_top_altitudes(izo,ime,nlayers)
             
           ! cosine of zenith angle
           czen = cosineza(xlati,xlong,idnow,jhour,izo,ime)

           ! Clouds Optical Thickness
           call cloud_optical_thickness(izo,ime)

           ! Calculation of mean Z0 and sensible heat flux and extra urban temper.
           ! and wind correction
           call mean_z0_shf_extra_urban_temp(jmon,zom,fland,izo,ime)

           ! Calculation of sensible and virtual heat fluxes in K.m/s
           call sv_heat_flux(izo,ime)

           ! Friction velocity from input (Option 0) and from Louis 82 (Option 1)
           call friction_velocity(izo,ime)

           ! Calculation of Boundary Layer Height using a simplified version
           ! of Cheinet 2003 with one thermal mixed with Troen-Mahrt 1986
           ! in stable cases.
           call boundary_layer_hght(izo,ime,np.eq.opt%nsho)

           ! Obukov length and related parameters
!           call obukov_length(izo,ime)
           call obukov_length(izo,ime,fland)

          ! Vertical turbulent diffusivity at top CHIMERE layers
!           call vertical_turbulent_diffusivity(izo,ime,nlayers)
           call vertical_turbulent_diffusivity(izo,ime,nlayers,fland)

           ! Let us show we are alive ...
           if(np.eq.opt%nsho) then
              write (*,888) idnow,d_usta(izo,ime),diag_misc%potf &
                   ,d_wsta(izo,ime),d_hght(izo,ime),d_atte(izo,ime) &
                   ,diag_misc%opdl,diag_misc%opdm,diag_misc%opdh &
                   ,diag_misc%topcld
888           format(i10,f7.3,f7.3,f7.3,f7.0,f7.3,f7.2,f7.2,f7.2,f7.0)

              write (90,'(a,25(1x,f7.1))') "al",al(1:nlevels)
              write (90,'(a,25(1x,f7.0))') "pr",pr(1:nlevels)
              write (90,'(a,25(1x,f7.2))') "uw",uw(1:nlevels)
              write (90,'(a,25(1x,f7.2))') "vw",vw(1:nlevels)
              write (90,'(a,25(1x,f7.2))') "te",te(1:nlevels)
              write (90,'(a,25(1x,f7.2))') "wi",wi(1:nlevels)
              write (90,'(a,25(1x,f7.2))') "po",po(1:nlevels)
           endif
	   
!******************************************************************	    
!***************** DEEP CONVECTION IMPLEMENTATION *****************
! Including calculation of deep convection	    
! Calculation of convection
!  - Inversion of the profiles
	    do ive=0,nlevels
               alc(ive) = al(nlevels-ive)  
               prc(ive) = pr(nlevels-ive)
               uvc(ive) = uw(nlevels-ive)
               vvc(ive) = vw(nlevels-ive)
               wwc(ive) = wwpa(nlevels-ive)
               tec(ive) = te(nlevels-ive)
               qrc(ive) = qr(nlevels-ive)
	       dvc(ive) = dv(nlevels-ive)
	       dqc(ive) = dq(nlevels-ive)
	    enddo	    
!  - Application of Tiedke Scheme provided by KNMI
! Units:
!     alc: THIS IS NOT REALLY geopotential height (m) but altitude AGL (m)
!     prc	pressure 	(Pa) 			[unit OK; no change]
!     tec	temperature 	(K) 			[unit OK; no change]
!     qrc	moisture 	(kg water/kg air) 	[specific humidity, OK]
!     wwc       vertical velocity (negative upward) (Pa s**-1)
!     dqc       div(q v) 	(kg water/kg air s**-1) [unit OK; no change]
!     dvc       div(v) 		(s**-1) 		[unit OK; no change]
!     evapo     Fq surf-air	(kg water/m**2 s**-1)	[unit OK; no change]
!
	    call convection(nlevels,alc,prc,tec,qrc,wwc,      &
	                    dqc,dvc,evapo,umflc,dmflc,euc,duc,edc,ddc)
!  - Inversion of updraught and downdraught mass fluxes
	    dpeu(0)=0.
	    dpdu(0)=0.
	    dped(0)=0.
	    dpdd(0)=0.
	    do ive=1,nlevels	    
               dpeu(ive) = euc(nlevels-ive+1)
               dpdu(ive) = duc(nlevels-ive+1)
               dped(ive) = edc(nlevels-ive+1)
               dpdd(ive) = ddc(nlevels-ive+1)
	    enddo

           ! CHIMERE Layer Averages
           call layave(nlevels,nlayers,al,uw,d_winz(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,vw,d_winm(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,te,d_temp(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,qr,d_sphu(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,de,d_airm(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,cl,d_clwc(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,dpeu,d_dpeu(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,dpdu,d_dpdu(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,dped,d_dped(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,dpdd,d_dpdd(izo,ime,:),d_alti(izo,ime,:))
           call layave(nlevels,nlayers,al,ww,  d_winw(izo,ime,:),d_alti(izo,ime,:))

           call layave(nlevels,nlayers,al,th,  d_tchi(izo,ime,:),d_alti(izo,ime,:))
! Wind correction in the surface layer
           d_winz(izo,ime,1) = diag_misc%awf*d_winz(izo,ime,1)
           d_winm(izo,ime,1) = diag_misc%awf*d_winm(izo,ime,1)

           ! Precipitations
!           d_topc(izo,ime)=m_lspc(izo,ime) + m_copc(izo,ime)
           d_topc(izo,ime)=m_lspc(izo,ime) 
     enddo !izo=1,nzonal
     enddo !ime=1,nmerid
     
! estimation of time step using CFL criteria
! conversion from entrainment flux to vertical speed = *1.3 for air density in kg/m3
  dtminu=1.e20
  dtminv=1.e20
  dtminw=1.e20
  dtminup_eu=1.e20
  dtminup_du=1.e20
  dtminup_ed=1.e20
  dtminup_dd=1.e20
  dxoveru=1.e20
  dxoverv=1.e20
  dzoverw=1.e20
  do ive=1,nlayers
  do ime=1,nmerid
  do izo=1,nzonal
     if(ive.eq.1)then
        layerdepth=d_alti(izo,ime,ive)
     else
        layerdepth=d_alti(izo,ime,ive)-d_alti(izo,ime,ive-1)
     endif
     if( d_winm(izo,ime,ive)/=0.0) dxoverv=cflmax*abs(ysize(izo,ime)/d_winm(izo,ime,ive))
     if( d_winz(izo,ime,ive)/=0.0) dxoveru=cflmax*abs(xsize(izo,ime)/d_winz(izo,ime,ive))
     if( d_winw(izo,ime,ive)/=0.0) dzoverw=cflmax*abs(layerdepth/d_winw(izo,ime,ive))
     if(dxoveru.le.dtminu)dtminu=dxoveru
     if(dxoverv.le.dtminv)dtminv=dxoverv
     if(dzoverw.le.dtminw)dtminw=dzoverw
     if(ideepconv.eq.1)then
   write(6,*)'ideepconv ',ideepconv     
        if(d_dpeu(izo,ime,ive).ne.0.0) then
            dzovereu=cflmax*abs(layerdepth/d_dpeu(izo,ime,ive))*1.3
        else
            dzovereu=1.e20
        endif
        if(dzovereu.le.dtminup_eu) then
          dtminup_eu=dzovereu
        endif

        if(d_dpdu(izo,ime,ive).ne.0.0) then
            dzoverdu=cflmax*abs(layerdepth/d_dpdu(izo,ime,ive))*1.3
        else
            dzoverdu=1.e20
        endif
        if(dzoverdu.le.dtminup_du) then
          dtminup_du=dzoverdu
        endif

        if(d_dped(izo,ime,ive).ne.0.0) then
            dzovered=cflmax*abs(layerdepth/d_dped(izo,ime,ive))*1.3
        else
            dzovered=1.e20
        endif
        if(dzovered.le.dtminup_ed) then
          dtminup_ed=dzovered
        endif

        if(d_dpdd(izo,ime,ive).ne.0.0) then
            dzoverdd=cflmax*abs(layerdepth/d_dpdd(izo,ime,ive))*1.3
        else
            dzoverdd=1.e20
        endif
        if(dzoverdd.le.dtminup_dd) then
          dtminup_dd=dzoverdd
        endif
    
     else
        dtminup_eu=dtminw
        dtminup_du=dtminw
        dtminup_ed=dtminw
        dtminup_dd=dtminw
     endif
  enddo
  enddo
  enddo
  dtmin=dtminu
  if(dtminv.lt.dtmin)dtmin=dtminv
  if(dtminw.lt.dtmin)dtmin=dtminw
  if(dtminup_eu.lt.dtmin)dtmin=dtminup_eu
  if(dtminup_du.lt.dtmin)dtmin=dtminup_du
  if(dtminup_ed.lt.dtmin)dtmin=dtminup_ed
  if(dtminup_dd.lt.dtmin)dtmin=dtminup_dd
  ! the time step to really use is nphour*ichemstep
  ! to force nphour, divided by ichemstep
  nphourm(itim)=nint(real(3600./dtmin/ichemstep))
!deb  write(6,*)itim,nint(3600./dtminu/ichemstep),nint(3600./dtminv/ichemstep)
!             nint(3600./dtminw/ichemstep),nphour_ref,nphourm(itim)

    if(mod(itim,12).eq.0)print*,'  >>  ',itim,'/',nhourrun,'h',' minimum.time.step: ',int(dtmin/60.),'mn', 'ichem ',ichemstep

     ! Writing to output file
     call write_data


  enddo !itim=1,nhourrun+1

  do itim=1,nhourrun
     nphourm_int(itim)=max(nphourm(itim),nphourm(itim+1))
     if(nphourm_int(itim).lt.nphour_ref)then
       cumnphour=cumnphour+nphour_ref
     else
       cumnphour=cumnphour+nphourm_int(itim)
     endif
  enddo
  nphourm_int(nhourrun+1)=nphourm_int(nhourrun) ! not used in CHIMERE
  
  print*,'Number of model integration steps:'
  print*,'  forced by user: ',nphour_ref*(nhourrun)
  print*,'  recommended to respect CFL: ',cumnphour
  tpercent=(real(cumnphour)/real((nphour_ref*(nhourrun))))*100.-100.
  print*,'  induced time increase: ',tpercent,'%'
  
  ! writing table for time step
  ncstat=nf90_put_var(d_id,d_nphourm_varid,nphourm_int)
  NCERR(__LINE__)

  call dealloc_diagmet_locals
  call dealloc_diagmet_globals

  ncstat=nf90_close(m_id)
  NCERR(__LINE__)
  ncstat=nf90_close(d_id)
  NCERR(__LINE__)

!#ifdef DEBUG_DIAGMET
  close(ifn_tst)
!#endif
!  close(ifn_theta)


  !**************************************************************************************

contains

  !****************************************************
  subroutine get_options
    implicit none

    call opfi(ifn_minf,fniMETINFOS,'f','o')
    read(unit=ifn_minf,nml=metoptions)

    

    u10m_req = (opt%w10m.eq.0)
    v10m_req = (opt%w10m.eq.0)
!       write(6,*)'v10m ',v10m_req,opt%w10m
    usta_req = (opt%usta.eq.0)
    sshf_req = (opt%flux.eq.0)
    slhf_req = (opt%flux.eq.0)
    pblh_req = (opt%pblh.eq.0)
    clol_req = (opt%clol.eq.0)
    clom_req = (opt%clom.eq.0)
    cloh_req = (opt%cloh.eq.0)
    cice_req = (opt%cice.eq.0).and.(opt%clol.eq.1.or.opt%clom.eq.1.or.opt%cloh.eq.1)
    rain_req = (opt%rain.eq.0).and.(opt%clol.eq.1.or.opt%clom.eq.1.or.opt%cloh.eq.1)
    soim_req = (opt%soim.eq.0)

    close(ifn_minf)

  ! Check MEGAN emissions flag
    swrd_req = .true.

  end subroutine get_options

  !****************************************************
  subroutine read_atts
    ! Read "history" attribute from input file
    ncstat=nf90_inquire_attribute(m_id,NF90_GLOBAL, 'history', len=hin_len)
    NCERR(__LINE__)
    ncstat=nf90_get_att(m_id,NF90_GLOBAL, 'history', history_in)
    NCERR(__LINE__)
    history_in=history_in(1:hin_len)
  end subroutine read_atts

  !****************************************************
  subroutine verif_req


#define NOTMAND(string) if (ncstat/=NF90_NOERR) call not_mand(string)
#define NOTOPT(string) if (ncstat/=NF90_NOERR) call not_opt(string)

    ncstat=nf90_inq_varid(m_id,'Times', m_times_varid)
    NOTMAND('Times')
    ncstat=nf90_inq_varid(m_id, 'lon',   m_lon_varid)
    NOTMAND('lon')
    ncstat=nf90_inq_varid(m_id, 'lat',   m_lat_varid)
    NOTMAND('lat')
    ncstat=nf90_inq_varid(m_id, 'alti',  m_alti_varid)
    NOTMAND('alti')
    ncstat=nf90_inq_varid(m_id, 'pres',  m_pres_varid)
    NOTMAND('pres')
    ncstat=nf90_inq_varid(m_id, 'temp',  m_temp_varid)
    NOTMAND('temp')
    ncstat=nf90_inq_varid(m_id, 'sphu',  m_sphu_varid)
    NOTMAND('sphu')
    ncstat=nf90_inq_varid(m_id, 'winz',  m_winz_varid)
    NOTMAND('winz')
    ncstat=nf90_inq_varid(m_id, 'winm',  m_winm_varid)
    NOTMAND('winm')
    ncstat=nf90_inq_varid(m_id, 'cliq',  m_cliq_varid)
    NOTMAND('cliq')
    ncstat=nf90_inq_varid(m_id, 'tem2',  m_tem2_varid)
    NOTMAND('tem2')
!    ncstat=nf90_inq_varid(m_id, 'copc',  m_copc_varid)
!    NOTMAND('copc')
    ncstat=nf90_inq_varid(m_id, 'lspc',  m_lspc_varid)
    NOTMAND('lspc')
    if (usta_req) then
       ncstat=nf90_inq_varid(m_id, 'usta',  m_usta_varid)
       NOTOPT('usta')
    end if
    if (v10m_req) then
       ncstat=nf90_inq_varid(m_id, 'u10m',  m_u10m_varid)
       NOTOPT('u10m')
       ncstat=nf90_inq_varid(m_id, 'v10m',  m_v10m_varid)
       NOTOPT('v10m')
    end if
    if (sshf_req) then
       ncstat=nf90_inq_varid(m_id, 'sshf',  m_sshf_varid)
       NOTOPT('sshf')
    end if
    if (slhf_req) then
       ncstat=nf90_inq_varid(m_id, 'slhf',  m_slhf_varid)
       NOTOPT('slhf')
    end if
    if (pblh_req) then
       ncstat=nf90_inq_varid(m_id, 'hght',  m_hght_varid)
       NOTOPT('hght')
    end if
    if (clol_req) then
       ncstat=nf90_inq_varid(m_id, 'clol',  m_clol_varid)
       NOTOPT('clol')
    end if
    if (clom_req) then
       ncstat=nf90_inq_varid(m_id, 'clom',  m_clom_varid)
       NOTOPT('clom')
    end if
    if (cloh_req) then
       ncstat=nf90_inq_varid(m_id, 'cloh',  m_cloh_varid)
       NOTOPT('cloh')
    end if
    if (cice_req) then
       ncstat=nf90_inq_varid(m_id, 'cice',  m_cice_varid)
       NOTOPT('cice')
    end if
    if (rain_req) then
       ncstat=nf90_inq_varid(m_id, 'rain', m_rain_varid)
       NOTOPT('rain')
    end if
    if (soim_req) then
       ncstat=nf90_inq_varid(m_id, 'soim',  m_soim_varid)
       NOTOPT('soim')
    end if

    ! Shortwave radiation, only needed for MEGAN biogenic emissions
    if (swrd_req) then
       ncstat=nf90_inq_varid(m_id, 'swrd',  m_swrd_varid)
       NOTOPT('swrd')
    end if



  end subroutine verif_req


  !****************************************************
  subroutine not_mand(vname)
    character*(*) :: vname
    print *
    print *, 'diagmet.F90 : meteo file does not contain'
    print *, 'the mandatory variable ',vname
    stop
  end subroutine not_mand
  
  !****************************************************
  subroutine not_opt(vname)
    character*(*) :: vname
    print *
    print *, 'diagmet.F90 : meteo file does not contain'
    print *, 'the optional variable ',vname
    print *, 'Please check your file METINFO-<model>.sed'
    stop
  end subroutine not_opt

  !****************************************************
  subroutine read_dims

    integer :: we,sn

    ! IDs
    ncstat=nf90_inq_dimid(m_id, 'Time',        m_time_dimid)
    NCERR(__LINE__)
    ncstat=nf90_inq_dimid(m_id, 'DateStrLen',  m_date_dimid)
    NCERR(__LINE__)
    ncstat=nf90_inq_dimid(m_id, 'west_east',   m_zonal_dimid)
    NCERR(__LINE__)
    ncstat=nf90_inq_dimid(m_id, 'south_north', m_merid_dimid)
    NCERR(__LINE__)
    ncstat=nf90_inq_dimid(m_id, 'bottom_top',  m_levels_dimid)
    NCERR(__LINE__)
    ! lengths
    ! global variables nzonal and nmerid, local variable nlevels
    ! are obtained from input file header
    ncstat=nf90_inquire_dimension(m_id, m_time_dimid,  len=m_times)
    NCERR(__LINE__)
    ncstat=nf90_inquire_dimension(m_id, m_date_dimid,  len=dl)
    NCERR(__LINE__)
    if (dl/=dlen) stop 'diagmet : date format error in input file'
    ncstat=nf90_inquire_dimension(m_id, m_zonal_dimid, len=we)
    NCERR(__LINE__)
    if (we/=nzonal) stop 'diagmet : WE dimension error in input file'
    ncstat=nf90_inquire_dimension(m_id, m_merid_dimid, len=sn    )
    NCERR(__LINE__)
    if (sn/=nmerid) stop 'diagmet : SN dimension error in input file'
    ncstat=nf90_inquire_dimension(m_id, m_levels_dimid,len=nlevels)
    NCERR(__LINE__)

    diag_misc%nlevels=nlevels

  end subroutine read_dims

  !****************************************************
  subroutine alloc_diagmet_locals

    ! allocate memory for input fields
    allocate(xlong (nzonal,nmerid))
    allocate(xlati (nzonal,nmerid))
    allocate(xsize (0:nzonal+1,0:nmerid+1))
    allocate(ysize (0:nzonal+1,0:nmerid+1))
    allocate(xbasx (0:nzonal+1,0:nmerid+1))
    allocate(xbasy (0:nzonal+1,0:nmerid+1))
    allocate(ybasx (0:nzonal+1,0:nmerid+1))
    allocate(ybasy (0:nzonal+1,0:nmerid+1))
    
    allocate(m_tem2(nzonal,nmerid))
    allocate(m_copc(nzonal,nmerid))
    allocate(m_lspc(nzonal,nmerid))
    allocate(m_clol(nzonal,nmerid))
    allocate(m_clom(nzonal,nmerid))
    allocate(m_cloh(nzonal,nmerid))
    allocate(m_hght(nzonal,nmerid))
    allocate(m_usta(nzonal,nmerid))
    allocate(m_u10m(nzonal,nmerid))
    allocate(m_v10m(nzonal,nmerid))
    allocate(m_sshf(nzonal,nmerid))
    allocate(m_slhf(nzonal,nmerid))
    allocate(m_soim(nzonal,nmerid))
  ! SW radiation for MEGAN
    allocate(m_swrd(nzonal,nmerid))
    allocate(m_alti(nzonal,nmerid,nlevels))
    allocate(m_pres(nzonal,nmerid,nlevels))
    allocate(m_temp(nzonal,nmerid,nlevels))
    allocate(m_sphu(nzonal,nmerid,nlevels))
    allocate(m_winz(nzonal,nmerid,nlevels))
    allocate(m_winm(nzonal,nmerid,nlevels))
    allocate(m_cliq(nzonal,nmerid,nlevels))
    allocate(m_cice(nzonal,nmerid,nlevels))
    allocate(m_rain(nzonal,nmerid,nlevels))
    ! deep convection
    allocate(m_winw(nzonal,nmerid,nlevels))
    allocate(m_winwpa(nzonal,nmerid,nlevels))
    allocate(m_divu(nzonal,nmerid,nlevels))
    allocate(m_divq(nzonal,nmerid,nlevels))
    ! allocate memory for local fields
    allocate(al(0:nlevels))
    allocate(pr(0:nlevels))
    allocate(qr(0:nlevels))
    allocate(cl(0:nlevels))
    allocate(uw(0:nlevels))
    allocate(vw(0:nlevels))
    allocate(ww(0:nlevels))
    allocate(wwpa(0:nlevels))
    allocate(te(0:nlevels))
    allocate(rh(0:nlevels))
    allocate(po(0:nlevels))
    allocate(de(0:nlevels))
    allocate(th(0:nlevels))
    allocate(wi(0:nlevels))
    allocate(op(0:nlevels))
    allocate(dv(0:nlevels))
    allocate(dq(0:nlevels))
    allocate(fland(nzonal,nmerid,nlduse))
    allocate(cac(nlayers))
    allocate(cbc(nlayers))
    allocate(zom(nlduse,nmonth))

    allocate(nphourm(nhourrun+1))
    allocate(nphourm_int(nhourrun+1))

  allocate(div(nzonal,nmerid,nlevels))
  allocate(divq(nzonal,nmerid,nlevels))
  allocate(thlayloc(nzonal,nmerid,nlevels))
  allocate(airmloc(0:nzonal+1,0:nmerid+1,nlevels+1))
  allocate(uwestg(nzonal,nmerid,nlevels))
  allocate(hwestg(nzonal,nmerid,nlevels))
  allocate(swestg(nzonal,nmerid,nlevels))
  allocate(ueastg(nzonal,nmerid,nlevels))
  allocate(heastg(nzonal,nmerid,nlevels))
  allocate(seastg(nzonal,nmerid,nlevels))
  allocate(usouthg(nzonal,nmerid,nlevels))
  allocate(hsouthg(nzonal,nmerid,nlevels))
  allocate(ssouthg(nzonal,nmerid,nlevels))
  allocate(unorthg(nzonal,nmerid,nlevels))
  allocate(hnorthg(nzonal,nmerid,nlevels))
  allocate(snorthg(nzonal,nmerid,nlevels))
  allocate(fluxw(nzonal,nmerid,nlevels))
  allocate(fluxe(nzonal,nmerid,nlevels))
  allocate(fluxs(nzonal,nmerid,nlevels))
  allocate(fluxn(nzonal,nmerid,nlevels))
  allocate(vfluxo(nzonal,nmerid,nlevels))
  allocate(vfluxi(nzonal,nmerid,nlevels,nlevels+1))
  allocate(dqc(0:nlevels))
  allocate(dvc(0:nlevels))
  allocate(alc(0:nlevels))
  allocate(prc(0:nlevels))
  allocate(qrc(0:nlevels))
  allocate(wwc(0:nlevels))
  allocate(uvc(0:nlevels))
  allocate(vvc(0:nlevels))
  allocate(tec(0:nlevels))
  allocate(dpeu(0:nlevels))
  allocate(dpdu(0:nlevels))
  allocate(dped(0:nlevels))
  allocate(dpdd(0:nlevels))
  allocate(umflc(0:nlevels))
  allocate(dmflc(0:nlevels))
  allocate(euc(nlevels))
  allocate(duc(nlevels))
  allocate(edc(nlevels))
  allocate(ddc(nlevels))

  end subroutine alloc_diagmet_locals

  !****************************************************
  subroutine dealloc_diagmet_locals

    ! deallocate memory for input fields
    deallocate(xlong)
    deallocate(xlati)
    deallocate(xsize)
    deallocate(ysize)
    deallocate(xbasx)
    deallocate(xbasy)
    deallocate(ybasx)
    deallocate(ybasy)
    
    deallocate(m_tem2)
    deallocate(m_copc)
    deallocate(m_lspc)
    deallocate(m_clol)
    deallocate(m_clom)
    deallocate(m_cloh)
    deallocate(m_hght)
    deallocate(m_usta)
    deallocate(m_u10m)
    deallocate(m_v10m)
    deallocate(m_sshf)
    deallocate(m_slhf)
    deallocate(m_soim)
  ! SW radiation for MEGAN
    deallocate(m_swrd)
    deallocate(m_alti)
    deallocate(m_pres)
    deallocate(m_temp)
    deallocate(m_sphu)
    deallocate(m_winz)
    deallocate(m_winm)
    deallocate(m_cliq)
    deallocate(m_cice)
    deallocate(m_rain)
    ! deep convection
    deallocate(m_winw)
    deallocate(m_winwpa)
    deallocate(m_divu)
    deallocate(m_divq)
    ! deallocate memory for local fields
    deallocate(al)
    deallocate(pr)
    deallocate(qr)
    deallocate(cl)
    deallocate(uw)
    deallocate(vw)
    deallocate(ww)
    deallocate(wwpa)
    deallocate(te)
    deallocate(rh)
    deallocate(po)
    deallocate(de)
    deallocate(th)
    deallocate(wi)
    deallocate(op)
    deallocate(fland)
    deallocate(cac)
    deallocate(cbc)
    deallocate(zom)
    deallocate(dv)
    deallocate(dq)
    deallocate(nphourm)
    deallocate(nphourm_int)

  deallocate(div)
  deallocate(divq)
  deallocate(thlayloc)
  deallocate(airmloc)
  deallocate(uwestg)
  deallocate(hwestg)
  deallocate(swestg)
  deallocate(ueastg)
  deallocate(heastg)
  deallocate(seastg)
  deallocate(usouthg)
  deallocate(hsouthg)
  deallocate(ssouthg)
  deallocate(unorthg)
  deallocate(hnorthg)
  deallocate(snorthg)
  deallocate(fluxw)
  deallocate(fluxe)
  deallocate(fluxs)
  deallocate(fluxn)
  deallocate(vfluxo)
  deallocate(vfluxi)
  deallocate(dqc)
  deallocate(dvc)
  deallocate(alc)
  deallocate(prc)
  deallocate(qrc)
  deallocate(wwc)
  deallocate(uvc)
  deallocate(vvc)
  deallocate(tec)
  deallocate(dpeu)
  deallocate(dpdu)
  deallocate(dped)
  deallocate(dpdd)
  deallocate(umflc)
  deallocate(dmflc)
  deallocate(euc)
  deallocate(duc)
  deallocate(edc)
  deallocate(ddc)
  
  end subroutine dealloc_diagmet_locals

  !****************************************************
  subroutine create_dims

    ncstat=nf90_def_dim(d_id,'Time',        NF90_UNLIMITED,d_time_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(d_id,'DateStrLen',  dlen,          d_date_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(d_id,'west_east',   nzonal,        d_zonal_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(d_id,'south_north', nmerid,        d_merid_dimid)
    NCERR(__LINE__)
    ncstat=nf90_def_dim(d_id,'bottom_top',  nlayers,       d_layers_dimid)
    NCERR(__LINE__)

  end subroutine create_dims

  !****************************************************
  subroutine define_out_vars

    ncstat=nf90_def_var(d_id,'Times', NFCHAR,(/d_date_dimid,d_time_dimid/), d_times_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id,'lon'  , NFLOAT,(/d_zonal_dimid,d_merid_dimid/), d_lon_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id,'lat'  , NFLOAT,(/d_zonal_dimid,d_merid_dimid/), d_lat_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id,'a_vcoord'  , NFLOAT,(/d_layers_dimid/), d_avcoord_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id,'b_vcoord'  , NFLOAT,(/d_layers_dimid/), d_bvcoord_varid)
    NCERR(__LINE__)
    
    ncstat=nf90_def_var(d_id,'nphourm'  , NFLOAT,(/d_time_dimid/), d_nphourm_varid)
    NCERR(__LINE__)
    


    allocate (d_dimids(3))
    d_dimids=(/d_zonal_dimid,d_merid_dimid, d_time_dimid/)
    ncstat=nf90_def_var(d_id, 'tem2', NFLOAT, d_dimids, d_tem2_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'atte', NFLOAT, d_dimids, d_atte_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'hght', NFLOAT, d_dimids, d_hght_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'usta', NFLOAT, d_dimids, d_usta_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'aerr', NFLOAT, d_dimids, d_aerr_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'obuk', NFLOAT, d_dimids, d_obuk_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'wsta', NFLOAT, d_dimids, d_wsta_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'sreh', NFLOAT, d_dimids, d_sreh_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'topc', NFLOAT, d_dimids, d_topc_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'w10m', NFLOAT, d_dimids, d_w10m_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'w10s', NFLOAT, d_dimids, d_w10s_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'soim', NFLOAT, d_dimids, d_soim_varid)
    NCERR(__LINE__)
    ! SW radiation at ground for MEGAN
    ncstat=nf90_def_var(d_id, 'swrd', NFLOAT, d_dimids, d_swrd_varid)
    NCERR(__LINE__)

    deallocate(d_dimids)

    allocate (d_dimids(4))
    d_dimids=(/d_zonal_dimid,d_merid_dimid,d_layers_dimid,d_time_dimid/)
    ncstat=nf90_def_var(d_id, 'alti', NFLOAT, d_dimids, d_alti_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'winz', NFLOAT, d_dimids, d_winz_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'winm', NFLOAT, d_dimids, d_winm_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'temp', NFLOAT, d_dimids, d_temp_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'sphu', NFLOAT, d_dimids, d_sphu_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'airm', NFLOAT, d_dimids, d_airm_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'kzzz', NFLOAT, d_dimids, d_kzzz_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'clwc', NFLOAT, d_dimids, d_clwc_varid)
    NCERR(__LINE__)
! lmbb deep convection
    ncstat=nf90_def_var(d_id, 'dpeu', NFLOAT, d_dimids, d_dpeu_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'dpdu', NFLOAT, d_dimids, d_dpdu_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'dped', NFLOAT, d_dimids, d_dped_varid)
    NCERR(__LINE__)
    ncstat=nf90_def_var(d_id, 'dpdd', NFLOAT, d_dimids, d_dpdd_varid)
    NCERR(__LINE__)
! lmbb deep convection
   ! theta for plume rise 
   ncstat=nf90_def_var(d_id, 'theta', NFLOAT, d_dimids, d_theta_varid)
    NCERR(__LINE__)
    
    deallocate(d_dimids)

    ncstat=nf90_put_att(d_id, d_lon_varid,'units','degrees_east')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id, d_lat_varid,'units','degrees_north')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id, d_avcoord_varid,'units','no_units')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id, d_bvcoord_varid,'units','no_units')
    NCERR(__LINE__)
    
    ncstat=nf90_put_att(d_id, d_nphourm_varid,'units','hour-1')
    NCERR(__LINE__)

    ncstat=nf90_put_att(d_id,d_tem2_varid,'units','K')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_atte_varid,'units','0-1')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_hght_varid,'units','m')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_usta_varid,'units','m/s')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_aerr_varid,'units','*')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_obuk_varid,'units','m')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_wsta_varid,'units','m/s')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_sreh_varid,'units','0-1')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_topc_varid,'units','kg/m^2')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_w10m_varid,'units','m/s')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_w10s_varid,'units','m/s')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_soim_varid,'units','m^3/m^3')
    NCERR(__LINE__)
  ! SW radiation at ground for MEGAN
    ncstat=nf90_put_att(d_id,d_swrd_varid,'units','W/m^2')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_alti_varid,'units','m')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_winz_varid,'units','m/s')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_winm_varid,'units','m/s')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_temp_varid,'units','K')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_sphu_varid,'units','kg/kg')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_airm_varid,'units','molec/cm3')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_kzzz_varid,'units','*')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_clwc_varid,'units','Kg/Kg')
    NCERR(__LINE__)
! lmbb deep convection
    ncstat=nf90_put_att(d_id,d_dpeu_varid,'units','kg/m2/s')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_dpdu_varid,'units','kg/m2/s')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_dped_varid,'units','kg/m2/s')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_dpdd_varid,'units','kg/m2/s')
    NCERR(__LINE__)
! lmbb deep convection
    ncstat=nf90_put_att(d_id, d_lon_varid,'long_name','Longitude')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id, d_lat_varid,'long_name','Latitude')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id, d_avcoord_varid,'long_name','A_sigma_coefficient')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id, d_bvcoord_varid,'long_name','B_sigma_coefficient')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id, d_nphourm_varid,'long_name','steps per hour')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_tem2_varid,'long_name','2m air temperature')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_atte_varid,'long_name','Cloud attenuation')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_hght_varid,'long_name','PBL height')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_usta_varid,'long_name','Frictional velocity')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_aerr_varid,'long_name','*')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_obuk_varid,'long_name','Obukov length')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_wsta_varid,'long_name','Wstar')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_sreh_varid,'long_name','Sfc rel. humidity')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_topc_varid,'long_name','Total precip')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_w10m_varid,'long_name','10m wind')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_w10s_varid,'long_name','Saltation wind')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_soim_varid,'long_name','Soil moisture')
    NCERR(__LINE__)
    ! SW radiation at ground for MEGAN
    ncstat=nf90_put_att(d_id,d_swrd_varid,'long_name','Shortwave radiation')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_alti_varid,'long_name','Altitude of layer')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_winz_varid,'long_name','Zonal wind')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_winm_varid,'long_name','Meridional wind')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_temp_varid,'long_name','Temperature')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_sphu_varid,'long_name','Specific humidity')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_airm_varid,'long_name','Air density')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_kzzz_varid,'long_name','Kz')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_clwc_varid,'long_name','Cloud wat. content')
    NCERR(__LINE__)
! lmbb deep convection
    ncstat=nf90_put_att(d_id,d_dpeu_varid,'long_name','Entrainment in updraft')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_dpdu_varid,'long_name','Detrainment in updraft')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_dped_varid,'long_name','Entrainment in downdraft')
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,d_dpdd_varid,'long_name','Detrainment in downdraft')
    NCERR(__LINE__)
! lmbb deep convection
    ncstat=nf90_put_att(d_id,d_theta_varid,'long_name','theta ')
    NCERR(__LINE__)

  end subroutine define_out_vars

  !****************************************************
  subroutine create_global_atts

    ncstat=nf90_put_att(d_id,NF90_GLOBAL,'Title',title)
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,NF90_GLOBAL,'Sub-title',subtitle)
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,NF90_GLOBAL,'Generating_process',generator)
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,NF90_GLOBAL,'Conventions',conventions)
    NCERR(__LINE__)
    ncstat=nf90_put_att(d_id,NF90_GLOBAL,'Domain',domain)
    NCERR(__LINE__)

#if defined(IFORT)

    history = &
         'File '                                                        // &
         cwd(1:len_trim(cwd)) // '/'//fnoMET(1:len_trim(fnoMET))        // &
         ' was generated on '                                           // &
         systime(1:len_trim(systime))                                   // &
         ' by '                                                         // &
         usrname(1:len_trim(usrname))                                   // &
         ' on '                                                         // &
         hname(1:len_trim(hname))                                       // &
         ' from input file '                                            // &
         cwd(1:len_trim(cwd))//'/'//fniMETEO(1:len_trim(fniMETEO))      //'\n'C
    history = history(1:len_trim(history)-1) // history_in(1:len_trim(history_in)-1)

#elif (defined(G95)+defined(PGI))

    history = &
         'File '                                                        // &
         cwd(1:len_trim(cwd)) // '/'//fnoMET(1:len_trim(fnoMET))        // &
         ' was generated on '                                           // &
         systime(1:len_trim(systime))                                   // &
         ' by '                                                         // &
         usrname(1:len_trim(usrname))                                   // &
         ' on '                                                         // &
         hname(1:len_trim(hname))                                       // &
         ' from input file '                                            // &
         cwd(1:len_trim(cwd))//'/'//fniMETEO(1:len_trim(fniMETEO))
    history = history(1:len_trim(history)-1) // history_in(1:len_trim(history_in)-1)

#else

    history='unknown'

#endif
    ncstat=nf90_put_att(d_id,NF90_GLOBAL,'history',history(1:len_trim(history)))
    NCERR(__LINE__)

  end subroutine create_global_atts

  !****************************************************
  subroutine init_optional
    ! Initialization of some optional variables
    implicit none

    integer :: izo,ime,ilev

    do ilev=1,nlevels
       do ime=1,nmerid
          do izo=1,nzonal
             m_cliq(izo,ime,ilev) = 0.
             m_rain(izo,ime,ilev) = 0.
             m_cice(izo,ime,ilev) = 0.
          end do
       end do
    end do
    do ime=1,nmerid
       do izo=1,nzonal
          m_soim(izo,ime) = soimdef
          ! SW radiation for MEGAN
          m_swrd(izo,ime) = 0.
       end do
    end do

  end subroutine init_optional

  !****************************************************
  subroutine read_data
    implicit none

    ! Times
    ncstat=nf90_get_var(m_id,m_times_varid,datebuf,(/1,itim/),(/dlen,1/))
    NCERR(__LINE__)
    
    !! 2D variables
    allocate(m_stvec(3))
    allocate(m_cntvec(3))
    m_stvec=(/1,1,itim/)
    m_cntvec=(/nzonal,nmerid,1/)
    !!! mandatory
    ncstat=nf90_get_var(m_id, m_tem2_varid, m_tem2, m_stvec,m_cntvec)
    NCERR(__LINE__)
!    ncstat=nf90_get_var(m_id, m_copc_varid, m_copc, m_stvec,m_cntvec)
!    NCERR(__LINE__)
    ncstat=nf90_get_var(m_id, m_lspc_varid, m_lspc, m_stvec,m_cntvec)
    NCERR(__LINE__)
    !!! optional
    if (clol_req) then
       ncstat=nf90_get_var(m_id, m_clol_varid, m_clol, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if
    if (clom_req) then
       ncstat=nf90_get_var(m_id, m_clom_varid, m_clom, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if
    if (cloh_req) then
       ncstat=nf90_get_var(m_id, m_cloh_varid, m_cloh, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if
    if (pblh_req) then
       ncstat=nf90_get_var(m_id, m_hght_varid, m_hght, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if
    if (usta_req) then
       ncstat=nf90_get_var(m_id, m_usta_varid, m_usta, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if
    if (v10m_req) then
       ncstat=nf90_get_var(m_id, m_u10m_varid, m_u10m, m_stvec,m_cntvec)
       NCERR(__LINE__)
       ncstat=nf90_get_var(m_id, m_v10m_varid, m_v10m, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if
    if (sshf_req) then
       ncstat=nf90_get_var(m_id, m_sshf_varid, m_sshf, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if
    if (slhf_req) then
       ncstat=nf90_get_var(m_id, m_slhf_varid, m_slhf, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if
    if (soim_req) then
       ncstat=nf90_get_var(m_id, m_soim_varid, m_soim, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if
    ! SW radiation for MEGAN
    if (swrd_req) then
       ncstat=nf90_get_var(m_id, m_swrd_varid, m_swrd, m_stvec,m_cntvec)
       NCERR(__LINE__)
    end if

    deallocate(m_stvec)
    deallocate(m_cntvec)

    !! 3D variables
    allocate(m_stvec(4))
    allocate(m_cntvec(4))
    m_stvec=(/1,1,1,itim/)
    m_cntvec=(/nzonal,nmerid,nlevels,1/)
    ! mandatory
    ncstat=nf90_get_var(m_id, m_alti_varid, m_alti, m_stvec, m_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_get_var(m_id, m_pres_varid, m_pres, m_stvec, m_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_get_var(m_id, m_temp_varid, m_temp, m_stvec, m_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_get_var(m_id, m_sphu_varid, m_sphu, m_stvec, m_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_get_var(m_id, m_winz_varid, m_winz, m_stvec, m_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_get_var(m_id, m_winm_varid, m_winm, m_stvec, m_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_get_var(m_id, m_cliq_varid, m_cliq, m_stvec, m_cntvec)
    NCERR(__LINE__)

    ! optional
    if (cice_req) then
       ncstat=nf90_get_var(m_id, m_cice_varid, m_cice, m_stvec, m_cntvec)
       NCERR(__LINE__)
    end if
    if (rain_req) then
       ncstat=nf90_get_var(m_id, m_rain_varid, m_rain, m_stvec, m_cntvec)
       NCERR(__LINE__)
    end if
    deallocate(m_stvec)
    deallocate(m_cntvec)
  end subroutine read_data

  !****************************************************
  subroutine check_date
    implicit none

    integer :: idc              ! current date,from netCDF input file
    integer :: mm5date2numeric  ! function to transform MM5 time into CHIMERE time

    idc=mm5date2numeric(datebuf)
    if (idc/=idnow) then
       print *,'*** diagmet : Date Problem with Meteo File: '
       print *,'*** Expected date: ',idnow,' File date: ',idc
       stop
    endif

  end subroutine check_date

  !****************************************************
  subroutine write_data
    ncstat=nf90_put_var(d_id,d_times_varid,datebuf,(/1,itim/),(/dlen,1/))
    NCERR(__LINE__)
    allocate(d_stvec(4))
    allocate(d_cntvec(4))
    d_stvec=(/1,1,1,itim/)
    d_cntvec=(/nzonal,nmerid,nlayers,1/)
    ncstat=nf90_put_var(d_id, d_alti_varid, d_alti,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_winz_varid, d_winz,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_winm_varid, d_winm,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_temp_varid, d_temp,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_sphu_varid, d_sphu,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_airm_varid, d_airm,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_kzzz_varid, d_kzzz,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_clwc_varid, d_clwc,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    
! ===== DEEP CONVECTION IMPLEMENTATION begin[7]     
    ncstat=nf90_put_var(d_id, d_dpeu_varid, d_dpeu,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_dpdu_varid, d_dpdu,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_dped_varid, d_dped,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_dpdd_varid, d_dpdd,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
! ===== DEEP CONVECTION IMPLEMENTATION end[7]     
    
   ! THETA plume rise 
    ncstat=nf90_put_var(d_id, d_theta_varid, d_tchi,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    
    deallocate(d_stvec)
    deallocate(d_cntvec)

    allocate(d_stvec(3))
    allocate(d_cntvec(3))
    d_stvec=(/1,1,itim/)
    d_cntvec=(/nzonal,nmerid,1/)
    ncstat=nf90_put_var(d_id, d_tem2_varid, d_tem2,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_atte_varid, d_atte,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_hght_varid, d_hght,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_usta_varid, d_usta,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_aerr_varid, d_aerr,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_obuk_varid, d_obuk,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_wsta_varid, d_wsta,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_sreh_varid, d_sreh,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_topc_varid, d_topc,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_w10m_varid, d_w10m,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_w10s_varid, d_w10s,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    ncstat=nf90_put_var(d_id, d_soim_varid, d_soim,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
   ! SW radiation at ground for MEGAN
    ncstat=nf90_put_var(d_id, d_swrd_varid, d_swrd,start=d_stvec,count=d_cntvec)
    NCERR(__LINE__)
    deallocate(d_stvec)
    deallocate(d_cntvec)
#ifdef DEBUG_DIAGMET

     write(ifn_tst,*)idnow 
     write(ifn_tst,*)'win_z ' , (((d_winz(izo,ime,ilay),izo=1,nzonal),ime=1,nmerid),ilay=1,nlayers) 
     write(ifn_tst,*)'win_m ', (((d_winm(izo,ime,ilay),izo=1,nzonal),ime=1,nmerid),ilay=1,nlayers) 
      write(ifn_tst,*)'temp ' ,(((d_temp(izo,ime,ilay),izo=1,nzonal),ime=1,nmerid),ilay=1,nlayers) 
      write(ifn_tst,*)'sphu ',(((d_sphu(izo,ime,ilay),izo=1,nzonal),ime=1,nmerid),ilay=1,nlayers)
       write(ifn_tst,*)'airm ' ,(((d_airm(izo,ime,ilay),izo=1,nzonal),ime=1,nmerid),ilay=1,nlayers) 
        write(ifn_tst,*)'clw ',(((d_clwc(izo,ime,ilay),izo=1,nzonal),ime=1,nmerid),ilay=1,nlayers) 
      write(ifn_tst,*)'alti ' ,(((d_alti(izo,ime,ilay),izo=1,nzonal),ime=1,nmerid),ilay=1,nlayers) 
         write(ifn_tst,*)'kzzz ' ,(((d_kzzz(izo,ime,ilay),izo=1,nzonal),ime=1,nmerid),ilay=1,nlayers) 
        write(ifn_tst,*)'tem2 '  ,((d_tem2(izo,ime),izo=1,nzonal),ime=1,nmerid) 
    write(ifn_tst,*)' sreh '    ,((d_sreh(izo,ime),izo=1,nzonal),ime=1,nmerid) 
   write(ifn_tst,*)'atte ', ((d_atte(izo,ime),izo=1,nzonal),ime=1,nmerid) 
    write(ifn_tst,*)'hgt '       ,((d_hght(izo,ime),izo=1,nzonal),ime=1,nmerid) 
     write(ifn_tst,*)'usta '        ,((d_usta(izo,ime),izo=1,nzonal),ime=1,nmerid) 
     write(ifn_tst,*)'aerr '        ,((d_aerr(izo,ime),izo=1,nzonal),ime=1,nmerid)
      write(ifn_tst,*)'obu '    ,((d_obuk(izo,ime),izo=1,nzonal),ime=1,nmerid) 
     write(ifn_tst,*)'wsta '       ,((d_wsta(izo,ime),izo=1,nzonal),ime=1,nmerid) 
     write(ifn_tst,*)'topc ',((d_topc(izo,ime),izo=1,nzonal),ime=1,nmerid)
#endif 

  end subroutine write_data

  !****************************************************

end program diagmet

